
`timescale 1ns / 1ps

module spi_regfile_testbench_v;

	// Inputs
	reg clk;
	reg reset;
	reg SCLK;
	reg SI;
	reg SS;

	// Outputs
	wire SO;

	// Instantiate the Unit Under Test (UUT)
	io_expander uut (
		.clk(clk), 
		.reset(reset), 
		.SCLK(SCLK), 
		.SI(SI), 
		.SO(SO), 
		.SS(SS)
	);

	initial begin
		// Initialize Inputs
		clk = 0;
		reset = 1;
		SCLK = 0;
		SI = 0;
		SS = 1;

		// Wait 100 ns for global reset to finish
		#100;
        
		// Add stimulus here

		reset = 0;
		#10;
		
		//----------------------------------------------------
		// Description sends 00000010 to say address 1 - write
		//----------------------------------------------------
		SS = 0; // active low selects the device
		#5;
		// send a byte
		// 1bit
		SI = 0; // first bit 1 = Read 0 = Write
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 2bit
		SI = 1; // lsb bit of address
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 3bit
		SI = 0; // 2nd bit of address
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 4bit
		SI = 0; // 3rd bit of address
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 5bit
		SI = 0; // 4th bit of address
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 6bit
		SI = 0; // 5th bit of address
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 7bit
		SI = 0; // 6th bit of address
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 8bit
		SI = 0; // 7th bit of address
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		//----------------------------------------------------
		// End 8 bit packet
		//----------------------------------------------------
		//----------------------------------------------------
		// Description sends 10101010 as data
		//----------------------------------------------------
		SS = 0; // active low selects the device
		#5;
		// send a byte
		// 1bit
		SI = 0; // lsb data bit
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 2bit
		SI = 1; // 2nd data bit 
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 3bit
		SI = 0; // 3rd data bit 
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 4bit
		SI = 1; // 4th data bit 
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 5bit
		SI = 0; // 5th data bit 
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 6bit
		SI = 1; // 6th data bit 
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 7bit
		SI = 0; // 7th data bit 
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 8bit
		SI = 1; // 8th data bit 
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		//----------------------------------------------------
		// End 8 bit packet
		//----------------------------------------------------
		
		//----------------------------------------------------
		// Description sends 00000000 to say address 1 - write
		//----------------------------------------------------
		SS = 0; // active low selects the device
		#5;
		// send a byte
		// 1bit
		SI = 0; // first bit 1 = Read 0 = Write
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 2bit
		SI = 0; // lsb bit of address
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 3bit
		SI = 0; // 2nd bit of address
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 4bit
		SI = 0; // 3rd bit of address
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 5bit
		SI = 0; // 4th bit of address
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 6bit
		SI = 0; // 5th bit of address
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 7bit
		SI = 0; // 6th bit of address
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 8bit
		SI = 0; // 7th bit of address
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		//----------------------------------------------------
		// End 8 bit packet
		//----------------------------------------------------
		//----------------------------------------------------
		// Description sends 00111100 as data
		//----------------------------------------------------
		SS = 0; // active low selects the device
		#5;
		// send a byte
		// 1bit
		SI = 0; // lsb data bit
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 2bit
		SI = 0; // 2nd data bit 
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 3bit
		SI = 1; // 3rd data bit 
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 4bit
		SI = 1; // 4th data bit 
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 5bit
		SI = 1; // 5th data bit 
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 6bit
		SI = 1; // 6th data bit 
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 7bit
		SI = 0; // 7th data bit 
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 8bit
		SI = 0; // 8th data bit 
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		//----------------------------------------------------
		// End 8 bit packet
		//----------------------------------------------------
		
		#50;
		
		//----------------------------------------------------
		// Description sends 00000001 to say address 0 -> read
		//----------------------------------------------------
		SS = 0; // active low selects the device
		#5;
		// send a byte
		// 1bit
		SI = 1; // first bit 1 = Read 0 = Write
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 2bit
		SI = 0; // lsb bit of address
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 3bit
		SI = 0; // 2nd bit of address
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 4bit
		SI = 0; // 3rd bit of address
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 5bit
		SI = 0; // 4th bit of address
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 6bit
		SI = 0; // 5th bit of address
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 7bit
		SI = 0; // 6th bit of address
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 8bit
		SI = 0; // 7th bit of address
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		//----------------------------------------------------
		// End 8 bit packet
		//----------------------------------------------------
		//----------------------------------------------------
		// READING
		//----------------------------------------------------
		SS = 0; // active low selects the device
		#5;
		// send a byte
		// 1bit
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 2bit
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 3bit
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 4bit
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 5bit
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 6bit
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 7bit
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 8bit
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		//----------------------------------------------------
		// End 8 bit packet
		//----------------------------------------------------
		
		#50;
		
		//----------------------------------------------------
		// Description sends 00000011 to say address 1 -> read
		//----------------------------------------------------
		SS = 0; // active low selects the device
		#5;
		// send a byte
		// 1bit
		SI = 1; // first bit 1 = Read 0 = Write
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 2bit
		SI = 1; // lsb bit of address
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 3bit
		SI = 0; // 2nd bit of address
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 4bit
		SI = 0; // 3rd bit of address
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 5bit
		SI = 0; // 4th bit of address
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 6bit
		SI = 0; // 5th bit of address
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 7bit
		SI = 0; // 6th bit of address
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 8bit
		SI = 0; // 7th bit of address
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		//----------------------------------------------------
		// End 8 bit packet
		//----------------------------------------------------
		//----------------------------------------------------
		// READING
		//----------------------------------------------------
		SS = 0; // active low selects the device
		#5;
		// send a byte
		// 1bit
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 2bit
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 3bit
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 4bit
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 5bit
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 6bit
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 7bit
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 8bit
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		//----------------------------------------------------
		// End 8 bit packet
		//----------------------------------------------------
		
		//----------------------------------------------------
		// Description sends 00000101 to say address 0 -> read
		//----------------------------------------------------
		SS = 0; // active low selects the device
		#5;
		// send a byte
		// 1bit
		SI = 1; // first bit 1 = Read 0 = Write
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 2bit
		SI = 0; // lsb bit of address
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 3bit
		SI = 1; // 2nd bit of address
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 4bit
		SI = 0; // 3rd bit of address
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 5bit
		SI = 0; // 4th bit of address
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 6bit
		SI = 0; // 5th bit of address
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 7bit
		SI = 0; // 6th bit of address
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 8bit
		SI = 0; // 7th bit of address
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		//----------------------------------------------------
		// End 8 bit packet
		//----------------------------------------------------
		//----------------------------------------------------
		// READING
		//----------------------------------------------------
		SS = 0; // active low selects the device
		#5;
		// send a byte
		// 1bit
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 2bit
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 3bit
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 4bit
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 5bit
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 6bit
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 7bit
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		// 8bit
		SCLK = 1;
		#25;
		SCLK = 0;
		#25;
		//----------------------------------------------------
		// End 8 bit packet
		//----------------------------------------------------
		
	end
      
	always
	begin
		#5 clk = ~clk;
	end
endmodule

